library my_rounded_button;

import 'package:flutter/material.dart';

const primaryGuardianColor = Color(0xFF00ABE9);
const lightGreyColor = Color(0xFFF2F2F2);
const mediumGreyColor = Color(0xFFB1B1B1);
const darkGreyColor = Color(0xFF707070);
const magentaColor = Color(0xFFD9308A);

class MyCustomButtonController extends ChangeNotifier {
  bool isAnimating = false;
  bool _isEnabled = false;

  void startAnimating() {
    this.isAnimating = true;
    notifyListeners();
  }

  void stopAnimating() {
    this.isAnimating = false;
    notifyListeners();
  }

  bool get isEnabled => _isEnabled;
  set isEnabled(bool enabled) {
    this._isEnabled = enabled;
    notifyListeners();
  }
}

class MyRoundedButton extends StatefulWidget {
  MyRoundedButton({
    Key? key,
    this.controller,
    required this.child,
    this.progressIndicator,
    required this.onPressed,
    this.width = double.infinity,
    this.color = magentaColor,
    this.hoverColor,
  }) : super(key: key);

  MyCustomButtonController? controller;
  final double width;
  final Widget child;
  final Widget? progressIndicator;
  final Function onPressed;
  final Color color;
  final Color? hoverColor;

  @override
  State<MyRoundedButton> createState() => _MyRoundedButtonState();
}

class _MyRoundedButtonState extends State<MyRoundedButton> {
  var _listener;
  bool _isHover = false;
  @override
  void initState() {
    // TODO: implement initState
    super.initState();
    if (widget.controller != null) {
      _listener = () => setState(() {});
      widget.controller!.addListener(_listener);
    }
  }

  @override
  void dispose() {
    // TODO: implement dispose
    if (widget.controller != null && _listener != null)
      widget.controller!.removeListener(_listener);
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: const EdgeInsets.fromLTRB(16, 8, 16, 8),
      child: Container(
          width: widget.width,
          decoration: BoxDecoration(borderRadius: BorderRadius.circular(10)),
          height: 50.0,
          child: MouseRegion(
            onEnter: (event) {
              setState(() {
                _isHover = true;
              });
            },
            onExit: (event) {
              setState(() {
                _isHover = false;
              });
            },
            child: MaterialButton(
              shape: RoundedRectangleBorder(
                borderRadius: BorderRadius.circular(10.0),
              ),
              color: widget.color,
              hoverColor: widget.hoverColor ?? widget.color,
              onPressed: () {
                if (widget.controller == null ||
                    (widget.controller != null &&
                        !widget.controller!.isAnimating)) {
                  widget.onPressed();
                }
              },
              child: widget.controller != null &&
                      widget.controller!.isAnimating &&
                      widget.progressIndicator != null
                  ? widget.progressIndicator
                  : widget.child,
            ),
          )),
    );
  }
}

class MyRoundedOutlineButton extends StatefulWidget {
  MyRoundedOutlineButton({
    Key? key,
    this.controller,
    required this.child,
    this.progressIndicator,
    required this.onPressed,
    this.borderColor = magentaColor,
    this.textStyle = const TextStyle(
      color: magentaColor,
      fontSize: 16.0,
    ),
  }) : super(key: key);

  MyCustomButtonController? controller;
  final Widget child;
  final Widget? progressIndicator;
  final Function onPressed;
  final Color borderColor;
  final TextStyle? textStyle;

  @override
  State<MyRoundedOutlineButton> createState() => _MyRoundedOutlineButtonState();
}

class _MyRoundedOutlineButtonState extends State<MyRoundedOutlineButton> {
  @override
  void initState() {
    // TODO: implement initState
    super.initState();
    if (widget.controller != null)
      widget.controller!.addListener(() => setState(() {}));
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      height: 50.0,
      width: double.infinity,
      margin: EdgeInsets.fromLTRB(16, 8, 16, 6),
      child: TextButton(
          onPressed: () {
            if (widget.controller != null && !widget.controller!.isAnimating) {
              widget.onPressed();
            }
          },
          style: ButtonStyle(
            shape: MaterialStateProperty.all<RoundedRectangleBorder>(
              RoundedRectangleBorder(
                  borderRadius: BorderRadius.circular(10.0),
                  side: BorderSide(
                    color: widget.borderColor,
                    width: 3.0,
                  )),
            ),
          ),
          child: widget.controller != null &&
                  widget.controller!.isAnimating &&
                  widget.progressIndicator != null
              ? widget.progressIndicator!
              : widget.child),
    );
  }
}
